	cpu Z80
	page 0

;******************************************************************************
; LC-80 MultiROM
;
; basiert auf SPIELE-EPROM von		Gunther Zielosko,
; reass. und neu zusammengesetzt	Volker Pohlers, 2003
; umsortiert und erweitert		Dr. Patrick Schfer, 2017-06-14
;------------------------------------------------------------------------------
; Start auf Adr. 1000h (Zusatz-ROM 3)
; ber die Tasten 0..9 sind 10 verschiedene Programme startbar:
; 	0:	NMI-Vektor setzen auf Registeranzeige
; 	1:	Logikspiel (Master Mind)
; 	2:	Geschicklichkeitsspiel (Reaktionstest)
; 	3:	Wrfelspiel
; 	4:	Spielautomat (Einarmiger Bandit)
; 	5:	Sterntaler (fallende Leuchtbalken)
; 	6:	Jagdspiel
; 	7:	U2716 (EPROM-Programmierung) mit eigenen Adressen
; 	8:	U2716 (EPROM-Programmierung) mit Standardwerten
; 	9:	RAM-Test
;
;******************************************************************************

	include	lc80
	org 01000h

;******************************************************************************
; Men
; ber die Tasten 0..9 sind 10 verschiedene Programme startbar.
;******************************************************************************
;
MENUE:		ld IX, MENUE_Text
		call DAK1		; ausgeben bis Taste gedrckt
		cp 00Ah			; ausserhalb 0..F?
		jr NC, MENUE		;  ja --> neuer Versuch
		ld HL, MENUE_Tab
		rlca
        	add A, L
        	ld L, A			; HL zeigt auf Tabelleneintrag
        	ld A, (HL)		; Sprungziel LSB	
        	inc HL		
        	ld H, (HL)		; Sprungziel MSB
        	ld L, A
        	jp (HL)
		
MENUE_Tab:	DW NMISETZEN		; 0: NMI-Vektor setzen auf Registeranzeige
		DW LOGIKSPIEL		; 1: Logikspiel (Master Mind)
		DW REAKTION		; 2: Geschicklichkeitsspiel (Reaktionstest)
		DW WUERFEL		; 3: Wrfelspiel
		DW BANDIT		; 4: Spielautomat (Einarmiger Bandit)
		DW TALER		; 5: Sterntaler (fallende Leuchtbalken)
		DW JAGD			; 6: Jagdspiel
		DW U2716_1		; 7: EPROM-Programmierung mit eigenen Adressen
		DW U2716		; 8: EPROM-Programmierung mit Standardwerten
		DW RAMTEST		; 9: RAM-Test
;
MENUE_Text:	db 05Dh, 0C2h, 0CEh, 021h, 04Fh, 0AEh	; "SPIEL?"

;******************************************************************************



;******************************************************************************
; Programm 0
; setzt den NMI-Vektor fr Registeranzeige
;******************************************************************************

NMISETZEN:	ld HL, NMI_Sprung
		ld DE, 02340h	
		ld BC, 00003h
		ldir
		jp 00000h		; Warmstart
;
NMI_Sprung:	DB 0C3h, 090h, 006h	; Registeranzeige		

;******************************************************************************



;******************************************************************************
; Programm 1
; Logikspiel (Master Mind)
; Autor: Gunther Zielosko
; "Hinweise zur Anwendung des Lerncomputers LC 80", Teil 3, Seite 49-56
;------------------------------------------------------------------------------
; Mastermind
; Der LC-80 gibt eine vierstellige Zahl vor, gebildet aus den Ziffern 0 .. 7,
; die mit mglichst wenigen Versuchen zu ermitteln ist. Dabei sind beliebige
; Wiederholungen einzelner Ziffern mglich (z. B. 1137).
;
; '+' drcken zum Start
;  -> Anzeige letzter Tipp | Anzahl der Versuche
; Tipp eingeben, '+' drcken zum Besttigen
;  -> Anzeige links Position und Wert richtig, rechts Position falsch aber Wert
;     richtig (z.B. 0P|1F - 1 richtig, aber falsche Stelle)
; '+' drcken zum nchsten Versuch
; bei 4Ps ist das Spiel gewonnen -> Fanfare
;******************************************************************************


LOGIK_Anz1	equ 02300h		; 6 Byte Anzeigespeicher
; 	Tipp und Anzahl der Versuche in 7-Segment-Code
;  	2305	2304	2303	2302	2301	2300
;	 T1	 T2	 T3	 T4	V(10)	V(1)
LOGIK_F_Wert	equ 02306h		; F = Anzahl der richtigen "Farben"
LOGIK_P_Wert	equ 02307h		; P = Anzahl der richtigen Positionen
LOGIK_ZufZahl	equ 02308h		; 4 Byte Zufallszahl (Z4..Z1)
LOGIK_Vers01	equ 0230Ch		; Anzahl der Versuche Einerstelle
LOGIK_Vers10	equ 0230Dh		; Anzahl der Versuche Zehnerstelle
Logik_Tipp_Wert	equ 0230Eh		; 4 Byte aktueller Tipp
LOGIK_Anz2	equ 02312h		; 6 Byte Anzeigespeicher
; 	P und F im 7-Segment-Code
;	2317	2316	2315	2314	2313	2312
;	 P	 'P'			 F	 'F'
Logik_Temp	equ 0231Ah		; temporre Kopie der Zufallszahl zum Vergleich


LOGIKSPIEL:	ld B, 01Fh		; Variablen lschen
		ld HL, LOGIK_Anz1	;  (2300 - 231F)
		ld A, 0
LOGIK_M1:	ld (HL), A
		inc HL
		djnz LOGIK_M1
		ld HL, LOGIK_ZufZahl	; untere zwei Zufallsziffern eintragen
		call LOGIK_ZUFAZ
; Laufschrift "LOGICSPIEL" ausgeben
LOGIK_M2:	ld C, 15		; 15 Zeichen
		ld IX, LOGIK_TXT	; IX zeigt auf LSB des auszugebenden Bereiches
LOGIK_M3:	ld B, 16		; 16x ausgeben (ca 160 ms)
LOGIK_M4:	call DAK2		; einmal ausgeben und bei Tastendruck Code in A zurckliefern
		cp 00Ah			; Taste '+' gedrckt?
		jr Z, LOGIK_M5		;  ja --> weiter
		djnz LOGIK_M4		;  sonst nochmal ausgeben
		dec IX			; Ausgabebereich verschieben (=scrollen)
		dec C
		jr NZ, LOGIK_M3		;  und ausgeben
		jr LOGIK_M2		; wenn komplett verschoben von vorne beginnen
; Spiel beginnen
LOGIK_M5:	ld HL,LOGIK_ZufZahl+2	; obere zwei Zufallszahlen eintragen
		call LOGIK_ZUFAZ
		ld A, 1			; Anzahl der Versuche auf 0 setzen
		ld (LOGIK_Vers01), A
LOGIK_M6:	ld HL,LOGIK_Vers01 +6	; Werte fr aktuellen Tipp und Nr des Versuches
		ld IX,LOGIK_Anz1 +6	; Anzeigebereich 1
		call LOGIK_ANZ		;  Werte in 7-Segment-Code umformen
		call DAK1		;  ausgeben bis Taste gedrckt
		cp 010h			; Taste '+' gedrckt?
		jr Z, LOGIK_M7		;  ja --> weiter
		call LOGIK_VERPRO	;  sonst Ziffer bernehmen und Anzeige schieben
		jr LOGIK_M6
; Tipp bernehmen
LOGIK_M7:	call LOGIK_VERGL	; P und F bestimmen
		ld A, (LOGIK_Vers01)	; Anzahl der Versuche erhhen
		add A,1
		daa			; in BCD wandeln
		ld (LOGIK_Vers01), A	; untere Stelle
		and 0F0h
		rrca
		rrca
		rrca
		rrca
		ld (LOGIK_Vers10), A	; obere Stelle
		ld A, 04Fh		; "P" in 2316h eintragen
		ld (LOGIK_Anz2 +4), A
		ld A, 04Eh		; "F" in 2312h eintragen
		ld (LOGIK_Anz2), A
		ld A, (LOGIK_F_Wert)	; F-Wert in 7-Segment-Form in 2313h laden
		call ONESEG
		ld (LOGIK_Anz2 +1), A
		ld A, (LOGIK_P_Wert)	; P-Wert in 7-Segment-Form in 2317h laden
		call ONESEG
		ld (LOGIK_Anz2 +5), A
		cp 02Bh			; P = '4'?
		jr Z, LOGIK_M8		;  nein --> weiter
LOGIK_M9:	ld IX, LOGIK_Anz2	; Anzeigebereich 2 mit P und F
		call DAK1		;  ausgeben bis Taste gedrckt
		jp LOGIK_M6
; gewonnen!
LOGIK_M8:	ld iy, LOGIK_NOTEN
		call music
		ld IX, LOGIK_Anz2	; Anzeigebereich 2 mit P und F
		call DAK1		;  ausgeben bis Taste gedrckt
		ld IX, LOGIK_Anz1	; Anzeigebereich 1 mit Zufallszahl und Anzahl der Versuche
		call DAK1		;  ausgeben bis Taste gedrckt
		jp LOGIKSPIEL		; ...und neue Runde
;
; Text "LOGICSPIEL    " als Laufschrift
		db 000h, 000h, 000h, 000h, 0C2h
		db 0CEh, 021h, 04fh, 0AEh, 0C6h
		db 021h, 0E6h, 0E7h, 0C2h
LOGIK_TXT:	db 000h, 000h, 000h, 000h, 000h, 000h
;
; Zufallzahlen erzeugen
; Aus dem Inhalt des Refresh-Registers werden zwei Zahlen im Bereich 0..7 generiert und
; in den Adressen (HL) und (HL+1) abgelegt.
LOGIK_ZUFAZ:	ld A, R			; A mit 2 Hex-Ziffern aus dem Refresh-Register laden
		and 077h		; Wertebereich auf 0..7 einschrnken
		rld			; unteres Nibble nach (HL)
		rrca			; oberes Nibble nach unten
		rrca
		rrca
		rrca
		inc HL
		rld			; unteres Nibble nach (HL+1)
		ret
;
; Variablen in 7-Segment-Code wandeln
; 6 aufeinanderfolgende Speicherstellen ab (HL-1) werden umgeformt und ab (IX-1) abgelegt
; Vorbereiten der Anzeige. Setzten des Punktes in Digit 4
LOGIK_ANZ:	ld B, 6			; 6 Stellen
LOGIK_M10:	dec HL
		dec IX
		ld A, (HL)		; Wert holen
		call ONESEG		; in 7-Segment-Code wandeln
		ld (IX+0), A		; und ablegen
		djnz LOGIK_M10
		ld HL, 02302h		; vierte Stelle
		set 4, (HL)		; bit 4 setzen um Dezimalpunkt einzuschalten
		ret
;
; Tipp-Ziffer bernehmen und Anzeige nach links schieben
; A->T4, T4->T3, T3->T2, T2-T1, T1 geht verloren
LOGIK_VERPRO:	ld B, 4			; vier Stellen
		ld HL, 0230Eh -1
LOGIK_M11:	inc HL			; nchste Stelle
		and 7			; Wert begrenzen
		rrd			; und im oberen Nibble von (HL) ablegen
		rr (HL)			; nach unten schieben
		rr (HL)
		rr (HL)
		rr (HL)			; (A enthlt nun das alte untere Nibble von (HL)
		djnz LOGIK_M11
		ret
;
; Aktuellen Tipp mit Zufallszahl vergleichen
; Anzahl bereinstimmender Stellen in P, Anzahl vorhandener Werte in F
LOGIK_VERGL:	call SOU1K		; Piepton
		ld A, 0
		ld (02306h), A		; P und F lschen
		ld (02307h), A
; Positionsvergleich durchfhren
		ld B, 4			; vier Stellen prfen
		ld DE, 0230Eh -1	; aktueller Tipp
		ld HL, 02308h -1	; Zufallszahl
LOGIK_M12:	inc DE			; nchste Stelle
		inc HL
		ld A, (DE)		; Tippziffer holen
		cp (HL)			; mit Zufallsziffer vergleichen
		jr NZ, LOGIK_M13
		ld A, (02307h)		; Treffer, P um 1 erhhen
		inc A
		ld (02307h), A
LOGIK_M13:	djnz LOGIK_M12
; Kopie der Zufallszahl Z1 bis Z4 auf 231ah bis 231dh ablegen (Z1' bis Z4')
		ld HL, 02308h		; Zufallszahl
		ld DE, 0231Ah		; Kopie
		ld BC, 4		; vier Stellen kopieren
		ldir
		ld B, 4
; Farbvergleich	durchfhren
		ld HL, 0230Eh		; aktueller Tipp
LOGIK_M14:	ld DE, 0231Ah		; Kopie der Zufallszahlen
		ld C, 4			; vier Stellen prfen
LOGIK_M15:	ld A, (DE)		; Zufallszahl holen
		cp (HL)			; mit Tipp vergleichen
		jr NZ, LOGIK_M16	;  passt nicht --> weiter
		ld A, (02306h)		; F um 1 erhhen
		inc A
		ld (02306h), A
		ld A,0FFh		; Zufallszahl mit 0FFh berschreiben
		ld (DE), A
		jr LOGIK_M17
LOGIK_M16:	inc DE			; nchste Tipp-Ziffer
		dec C
		jr NZ, LOGIK_M15
LOGIK_M17:	inc HL			; nchste Zufallszahl
		djnz LOGIK_M14
		ld HL, 02307h		; Adresse von P in HL
		ld A, (02306h)		; F in A laden
		sub (HL)		; und P subtrahieren
		ld (02306h), A
		ret
;
;
LOGIK_NOTEN:	db 00bh, 004h, 00fh, 004h
		db 012h, 004h, 017h, 010h
		db 080h
;
;******************************************************************************



;******************************************************************************
; Programm 2
; Geschicklichkeitsspiel (Reaktionstest)
; Autor: Gunther Zielosko
; "Hinweise zur Anwendung des Lerncomputers LC 80", Teil 1, Seite 77-84
;  (angepasst fr ROM-Ablauf)
;------------------------------------------------------------------------------
; Nach dem Programmstart erscheint nach ca. 1 sec. im rechten Feld des Displays
; eine Zufallsziffer (0 .. F). Innerhalb von wiederum etwa 1 sec, mssen wir
; die dazugehrige Taste finden und bettigen. Wenn das gelungen ist, wird in
; den beiden linken Feldern der Treffer angezeigt (mit 01) und ein kurzer Ton
; erzeugt. Die nchste Ziffer erscheint. Bei richtiger Tastenbettigung piept
; es wieder und die Trefferanzeige wird auf 02 erhht. Usw.
; Nach jeder Zufallszahl wird die Zeit zum Finden und Drcken der Taste krzer.
; Die Treffersumme bis zum automatischen Ende des Spieles ist ein "Dokument"
; der erreichten Fertigkeiten. Die bei mehreren Spieldurchlaufen jeweils
; erreichte Hchstpunktzahl wird als "Rekord" gespeichert. Dieser Rekord wird
; bei Spielende auf den beiden rechten Feldern des Display angezeigt. Wird der
; Rekord berboten, ertnt eine Siegesfanfare, wird er verfehlt, ist der
; Computer "traurig" und zeigt das auch.
;******************************************************************************

REAKT_Treffer	equ 020FBh
REAKT_ZufZahl	equ 020FCh
REAKT_Anzeige	equ 020FDh			; 6 Bytes
REAKT_Rekord	equ 02104h


REAKTION:	ld A, 050h
		ld (REAKT_Rekord), A		; Rekord auf 50 initialisieren
REAKTION1:	ld HL, 0
		ld (REAKT_Treffer), HL		; Treffer und Zufallszahl lschen
		ld (REAKT_Anzeige), HL		; Anzeige lschen
		ld (REAKT_Anzeige+2), HL
		ld (REAKT_Anzeige+4), HL
		ld B, 255			; 255x ausgeben (ca 2,5 s)
		ld IX, REAKT_Anzeige		; leere Anzeige
REAK_M1:	call DAK2			; einmal ausgeben und bei Tastendruck Code in A zurckliefern
		djnz REAK_M1
; Spielbeginn
		ld C, 255			; wir starten mit 2,5 s Anzeigedauer
; nchste Runde
REAK_M2:	ld A, R				; Inhalt des Refresh-Registers holen
		and 00Fh			; untere Stelle isolieren
		ld (REAKT_ZufZahl), A		; und als Zufallszahl nehmen
		call ONESEG			; in 7-Segment-Code wandeln
		ld (REAKT_Anzeige), A		; und in die rechte Stelle der Anzeige schreiben
		ld B, C				; Anzeigedauer
REAK_M3:	ld A, 033h			; Tastendruck ungltig machen
		ld HL, REAKT_ZufZahl
		call DAK2			; einmal ausgeben und bei Tastendruck Code in A zurckliefern
		add A, lo(REAK_Tasten)
		ld E, A
		ld D, hi(REAK_Tasten)
		ld A, (DE)			; von dieser Adresse die Tastenwertigkeit holen
		cp (HL)				; richtige Taste gedrckt?
		jr Z, REAK_M6			;  ja --> weiter
		djnz REAK_M3			;  sonst neu versuchen
; Zeit ist abgelaufen
REAK_M4:	ld A, C				; Anzeigedauer
		sub 2				; um 20 ms verkrzen
		jr C, REAK_M5			; schon unter Null?
		ld C,  A			; Anzeigedauer aktualisieren und nchste Runde
		jr REAK_M2
; Spiel zuende
REAK_M5:	jp REAK_M8
; Tastendruck war richtig
REAK_M6:	push BC				; Restlaufzeit und Anzeigedauer sichern
		ld C, 025h			; Frequenz
		ld HL, 020h			; Dauer
		call sound			; Ton erzeugen
		ld A, (REAKT_Treffer)
		inc A				; Trefferzahl erhhen
		daa
		ld (REAKT_Treffer), A
		ld HL, REAKT_Anzeige+4
		call TWOSEG
		pop BC
; Trefferzahl fr die Dauer der Restlaufzeit (B) anzeigen 
REAK_M7:	ld A, 0
		ld (REAKT_Anzeige), A		; Zufallszahl ausblenden  (gefllt mir so besser --ps)
		call DAK2			; einmal ausgeben und bei Tastendruck Code in A zurckliefern
		djnz REAK_M7
		jp REAK_M4
; Spiel zuende
REAK_M8:	ld A, (REAKT_Treffer)		; Trefferzahl
		ld B,  A
		ld A, (REAKT_Rekord)
		cp B				; ber Rekord?
		jr C, REAK_M10			;  nein --> weiter
		ld HL, REAKT_Anzeige
		call TWOSEG			; Inhalt von A (Rekord) in die rechten beiden Anzeigestellen
; Heulton erzeugen
		ld C, 001h			; Frequenz
REAK_M9:	ld HL, 002h			; Dauer
		call sound			; Ton erzeugen
		inc C				; Frequenz erhhen
		jr NZ, REAK_M9
		jp REAK_M11
; Rekord bertroffen!
REAK_M10:	ld A, (REAKT_Treffer)
		ld (REAKT_Rekord), A		; Trefferzahl ist neuer Rekord
		ld HL, REAKT_Anzeige
		call TWOSEG			; Inhalt von A (Rekord) in die rechten beiden Anzeigestellen
		push IX
		ld iy, REAK_Noten
		call music			; Siegeshymne abspielen
		pop IX
; Trefferzahl anzeigen
REAK_M11:	call DAK1			; Trefferzahl und Rekord ausgeben bis Taste gedrckt
		jp REAKTION1			; Neustart
;
;
REAK_Tasten:	db 000h, 000h, 000h, 000h	; EX, ST, LD, (leer)
		db 000h, 001h, 002h, 003h	; 0, 1, 2, 3
		db 004h, 005h, 000h, 007h	; 4, 5, +, 7
		db 008h, 009h, 00eh, 00bh	; 8, 9, E, B
		db 00ch, 00dh, 00ah, 00fh	; C, D, A, F
		db 0ffh, 0ffh, 006h, 0ffh	; ADR, DAT, 6, -
;
REAK_Noten:	db 00bh, 004h
		db 00fh, 004h
		db 012h, 004h
		db 017h, 010h
		db 080h
;
;******************************************************************************



;******************************************************************************
; Programm 3
; Wrfelspiel
; Autor: Gunther Zielosko
; s. "Hinweise zur Anwendung des Lerncomputers LC 80", Teil 1, Seite 41-45
;  (angepasst fr ROM-Ablauf)
;------------------------------------------------------------------------------
; Die '-'-Taste schaltet das Display dunkel, der Wrfel rollt.
; Beim Bettigen der '+'-Taste ertnt ein kurzer Piepton, danach rollt die
; Zufallszahl (1..6) von rechts nach links ein um auf der 1. Stelle des Display
; liegenzubleiben bis wieder '-' gedrckt wird.
;******************************************************************************

WUERFEL_Text	equ 2100h

WUERFEL:	ld HL, WUERFEL_Text0		; Anzeige initialisieren
		ld DE, WUERFEL_Text
		ld BC, 12
		ldir
WUERFEL_START:	ld IX, WUERFEL_Text
WUERFEL_START1:	call DAK1			; Anzeigen von 2100...2105
		cp 011h				; "-" - Taste gedrckt?
		jr NZ, WUERFEL_START1		;  nein --> warten
		ld IX, WUERFEL_Text+6
; wir zhlen solange von 1 bis 6 bis jemand '+' drckt
WUERFEL_ZAHL1:	call DAK2			; Anzeigen von 2106...210B
		ld C, 021h			; "1" in 7-Segment-Darstellung
		cp 00Ah				; "+" - Taste?
		jp Z, WUERFEL_WAHL
		jp WUERFEL_ZAHL2
WUERFEL_ZAHL2:	call DAK2
		ld C, 0CDh			; "2"
		cp 00Ah				; "+" - Taste?
		jp Z, WUERFEL_WAHL
		jp WUERFEL_ZAHL3
WUERFEL_ZAHL3:	call DAK2
		ld C, 0ADh			; "3"
		cp 00Ah				; "+" - Taste?
		jp Z, WUERFEL_WAHL
		jp WUERFEL_ZAHL4
WUERFEL_ZAHL4:	call DAK2
		ld C, 2Bh			; "4"
		cp 00Ah				; "+" - Taste?
		jp Z, WUERFEL_WAHL
		jp WUERFEL_ZAHL5
WUERFEL_ZAHL5:	call DAK2
		ld C, 0AEh			; "5"
		cp 00Ah				; "+" - Taste?
		jp Z, WUERFEL_WAHL
		jp WUERFEL_ZAHL6
WUERFEL_ZAHL6:	call DAK2
		ld C, 0EEh			; "6"
		cp 00Ah				; "+" - Taste?
		jp Z, WUERFEL_WAHL
		jp WUERFEL_ZAHL1
; aktuellen Zhlwert ausgeben
WUERFEL_WAHL:	ld A, C
		ld (WUERFEL_Text+5), A
		ld C, 025h			; Frequenz
		ld HL, 00100h			; Dauer
		call sound			; Ton erzeugen
		ld IX, WUERFEL_Text+6
		ld C, 6
WUERFEL_anz1:	ld B, 2				; Schnelligkeit der Laufschrift
WUERFEL_anz2:	call DAK2			; Laufschrift und Anzeige
		djnz WUERFEL_anz2
		dec IX
		dec C
		jr NZ, WUERFEL_anz1
		jp WUERFEL_START

WUERFEL_Text0:	db 000h, 000h, 000h, 000h, 000h, 0E7h	; Anzeige der "0" bei Beginn
		db 000h, 000h, 000h, 000h, 000h, 000h

;******************************************************************************



;******************************************************************************
; Programm 4
; Spielautomat (Einarmiger Bandit)
; Autor: Gunther Zielosko
; "Hinweise zur Anwendung des Lerncomputers LC 80", Teil 3, Seite 57-64
;------------------------------------------------------------------------------
; Drei rollendende Ziffern erscheinen. Beim Drcken von 1 wird die erste
; Stelle, bei 2 die zweite und bei 3 die dritte gestoppt. Gewinne werden
; rechts angezeigt, mglich sind 3 oder 9 Spielmarken, die einen Wert von je
; 0,10 M symbolisieren.
; Das Gewinnschema ist einfach:
;  bei Gleichheit der 1. und 2. Ziffer 3 (0,30 M),
;  bei Gleichheit der 1., 2. und 3. Ziffer 9 (0,90 M).
; Mit der Wahl der mglichen Ziffern (1 ... 6), dem Gewinnschema und der bei
; jedem Spiel symbolisch einzuzahlenden Spielmarke (0,10 M) ergibt sich
; statistisch, da 2/3 der eingezahlten Betrge als Gewinn wieder ausgezahlt
; werden. Der Rest bleibt im "Automaten". Zur Demonstration dieses Verhaltens
; wurde ein kleiner "Buchhalter" mit einprogrammiert, der bei stehenden Ziffern
; ber + abgefragt werden kann.
; Der Automat beginnt mit einem Startkapital von 10,00 M. Bei jedem Spiel
; kommen 0,10 M hinzu, jeder Gewinn wird automatisch abgezogen.
; Mit 0 beginnt ein neues Spiel - nicht vergessen, einen Groschen einzuzahlen!
; Viel Glck!
;******************************************************************************

BANDIT_Anz1	equ 02200h		; 6 Byte Anzeigespeicher
;	2205	2204	2203	2202	2201	2200
;	 Z1	 Z2	 Z3			  G
; 	  rollende Ziffern			Gewinn
BANDIT_Gew	equ 02206h		; Gewinn
BANDIT_Z3	equ 02209h		; Ziffern
BANDIT_Z2	equ 0220Ah
BANDIT_Z1	equ 0220Bh
BANDIT_n	equ 02211h		; Anzahl der noch rollenden Ziffern
BANDIT_Gut01	equ 02216h		; Guthaben Groschen
BANDIT_Gut10	equ 02217h		; Guthaben Mark
BANDIT_Anz2	equ 023F2h		; Standard-Anzeigespeicher
;	23F7	23F6	23F5	23F4	23F3	23F2
;	    Mark    .	   Groschen


BANDIT:		ld HL, 01000h		; 10,00 M in Bank laden
		ld (BANDIT_Gut01), HL
BANDIT_M1:	ld A,03			; alle drei Walzen drehen
		ld (BANDIT_n), A
; Speicherbereich von 2200 bis 220C lschen
		ld B, 0Ch
		ld HL, BANDIT_Anz1
		ld A, 0
BANDIT_M2:	ld (HL), A
		inc HL
		djnz BANDIT_M2
; Spielbeginn
		ld IX, BANDIT_Anz1	; Anzeigebereich 1
BANDIT_M3:	call BANDIT_rotanz
		cp 005h			; '1' gedrckt?
		jr NZ, BANDIT_M3	;  --> nein
		call BANDIT_stop	;  sonst 1. Ziffer Stop
BANDIT_M4:	call BANDIT_rotanz
		cp 006h			; '2' gedrckt?
		jr NZ, BANDIT_M4	;  --> nein
		call BANDIT_stop	;   sonst 2. Ziffer Stop
BANDIT_M5:	call BANDIT_rotanz
		cp 007h			; '3' gedrckt?
		jr NZ, BANDIT_M5	;  --> nein
		call SOU1K
		call BANDIT_vergl
; Wenn Gewinn <> 0, dann B mit Gewinnzahl (3 oder 9) laden und B-mal Wechselton erzeugen
		ld A, (BANDIT_Gew)
		cp 0
		jr Z, BANDIT_M7
		ld B, A
BANDIT_M6:	push BC
		call SOU1K
		call SOU2K
		pop BC
		djnz BANDIT_M6
;
BANDIT_M7:	call BANDIT_bilanz
BANDIT_M8:	call BANDIT_anz
		cp 004h			; '0' gedrckt?
		jr Z, BANDIT_M1		;  --> Neues Spiel?
		cp 00Ah			; '+' gedrckt?
		jr NZ, BANDIT_M8	;  --> nein
; Anzeige des Bankguthabens
		ld DE, (BANDIT_Gut01)	; Bankguthaben als "Adresse" darstellen
		call ADRSDP
		ld HL, 0
		ld (BANDIT_Anz2), HL	; untere zwei Stellen lschen
		ld HL, BANDIT_Anz2 +4
		set 4, (HL)		; Dezimalpunkt einschalten
		ld IX, BANDIT_Anz2
BANDIT_M9:	call DAK1
		cp 000h			; '0' gedrckt?
		jr NZ, BANDIT_M9	;  --> nein
		jr BANDIT_M1		;  sonst neues Spiel
;
; Erzeugen der 3 rollenden Ziffern
BANDIT_rotanz:	ld B, (IX+11h)		; Anzahl der noch rollenden Ziffern (2211)
		ld HL, BANDIT_Z3	; hinterste Ziffer
BANDIT_r1:	ld A, (HL)		; Wert holen
		inc A			;  und erhhen
		cp 7			; berlauf?
		jr NZ, BANDIT_r2	;  --> nein, weiter
		ld A, 1			;  sonst rckstelle auf 1
BANDIT_r2:	ld (HL), A		; zurckschreiben
		inc HL			; nchste Walze
		djnz BANDIT_r1
;
; bertragen der 3 rollenden Ziffern in den 7-Segment-Code und anzeigen
BANDIT_anz:	ld B, 3			; drei Ziffern
		ld HL, BANDIT_Z3
		ld DE, BANDIT_Anz1 +3
BANDIT_r3:	ld A, (HL)		; Ziffer holen
		call ONESEG		;  in 7-Segment-Code wandeln
		ld (DE), A		;  und in  im Anzeigebereich 1 eintragen
		inc HL
		inc DE
		djnz BANDIT_r3
		call DAK2		; fr 10 ms anzeigen
		ret
;
; Herabsetzen der Anzahl der rotierenden Ziffern
BANDIT_Stop:	ld A, (BANDIT_n)
		dec A
		ld (BANDIT_n), A
		call SOU1K
		ret
;
; Prfen auf Gewinn
BANDIT_vergl:	ld HL, BANDIT_Z1	; Vergleich der 1. mit der 2. Ziffer
		ld A, (HL)
		dec HL
		cp (HL)
		jr NZ, BANDIT_v1
		ld A, 3			; Gewinn = 3, wenn 1. und 2. Ziffer gleich
		ld (BANDIT_Gew), A
		call ONESEG		;  in 7-Segment-Code wandeln
		ld (BANDIT_Anz1), A
		ld A, (HL)		; Vergleich der 2. mit der 3. Ziffer
		dec HL
		cp (HL)
		jr NZ, BANDIT_v1
		ld A, 9			; Gewinn = 9, wenn 1., 2. und 3. Ziffer gleich
		ld (BANDIT_Gew), A
		call ONESEG		;  in 7-Segment-Code wandeln
		ld (BANDIT_Anz1), A
BANDIT_v1:	ret
;
; Bankguthaben aktualisieren
BANDIT_bilanz:	ld A, (BANDIT_Gew)	; Gewinnziffer 4x links verschieben (aus 09 wird 90)
		rlca
		rlca
		rlca
		rlca
		and A, 0F0h
		ld D, A			; Ergebnis in Register D laden
		ld A, (BANDIT_Gut01)
		sub D			; Gewinn vom Guthaben (Groschen) abziehen
		daa
		ld (BANDIT_Gut01), A
		jr NC, BANDIT_b1
		ld A, (BANDIT_Gut10)
		sub 1			; bei bertrag Markbetrag um 1 vermindern
		daa
		ld (BANDIT_Gut10), A
BANDIT_b1:	ld A, (BANDIT_Gut01)
		add A,10h
		daa			; Spieleinsatz (10) zu Guthaben (Groschen) addieren
		ld (BANDIT_Gut01), A
		jr NC, BANDIT_b2
		ld A, (BANDIT_Gut10)
		add A,1			; bei bertrag Markbetrag um 1 erhhen
		daa
		ld (BANDIT_Gut10), A
BANDIT_b2:	ret

;******************************************************************************



;******************************************************************************
; Programm 5
; Sterntaler (fallende Leuchtbalken)
; Autor: Gunther Zielosko
; "Hinweise zur Anwendung des Lerncomputers LC 80", Teil 3, Seite 64-69
;------------------------------------------------------------------------------
; Das Display stellt das "Spielfeld" dar, auf dem es "Taler" regnet. Letztere
; werden durch zufllig nach unten fallende Leuchtbalken der 7-Segment-Anzeigen
; symbolisiert. Leider fallen die Taler nach unten durch und sind dann durch
; nichts mehr zu erwischen. Glcklicherweise gibt es einen "Topf", durch einen
; Leuchtpunkt gebildet, der sich ber die Tasten 0 und 3 hin und her bewegen
; lt. Mit dem knnen wir die herabfallenden Taler auffangen, wenn wir ihn in
; die entsprechende Position rechts neben dem Leuchtbalken gebracht haben.
; Das Ganze wird im Spielverlauf immer komplizierter und die "aufgefangenen
; Taler" werden mit einem Ton quittiert. Gezhlt werden sie auch, so da am
; Spielende jeder wei, wie geschickt er gespielt hat.
; Ein neues Spiel beginnt, wenn die Taste + gedrckt wird.
;******************************************************************************

TALER_Anz1	equ 02200h		; 6 Byte Anzeigebereich (Spielfeld)
TALER_Punkte01	equ 02208h		; Punktzhler (Einer, Zehner)
TALER_Punkte10	equ 02209h		; Punktzhler (Hunderter, Tausender)
TALER_Pos	equ 02220h		; aktuelle Position des Fngers
TALER_Geschw	equ 02222h		; aktuelle Geschwindigkeit der fallenden Balken
TALER_Anz2	equ 023F2h		; Standard-Anzeigespeicher


TALER:		ld B, 010h		; Lschen des Anzeige- und Speicherbereiches
		ld HL, 02210h
TALER_h1:	ld (HL),0
		dec HL
		djnz TALER_h1
		ld IX, TALER_Anz1
		ld (HL), 010h		; Punkt setzen
		ld A, L
		ld (TALER_Pos), A	; Position des Punktes
		ld A, 28		; Anfangsgeschwindigkeit
		ld (TALER_Geschw), A
; Beginn des Spieles
TALER_h2:	ld D, 16		; Dauer einer Geschwindigkeitsstufe (war 32)
TALER_h3:	call TALER_Sterne	; Taler fallen lassen
		ld A, (TALER_Geschw)
		ld B,  A		; Verweildauer der Taler
TALER_h4:	push BC
		call TALER_Faenger	; Fnger positionieren
		pop BC
		djnz TALER_h4
		dec D
		jr NZ, TALER_h3
		ld A, (TALER_Geschw)	; Geschwindigkeit erhhen
		sub 2			;  (Stufen 01, 02, 04 mglich)
		jr Z, TALER_h5		; Spielende?
		ld (TALER_Geschw), A
		jr TALER_h2
; Spielende
TALER_h5:	ld DE, (TALER_Punkte01)
		call ADRSDP		; Punktestand in Anzeigebereich 2
		ld IX, TALER_Anz2
TALER_h6:	call DAK1		; anzeigen bis
		cp 010h			; '+' gedrckt?
		jr Z, TALER		;  ja --> neues Spiel
		jr TALER_h6
;
; Auf Tastendruck prfen und Position des Fngers anpassen
TALER_Faenger:	ld A, (TALER_Pos)	; Position des Fngers
		ld L, A			; als HL-Wert aufbereiten
		ld H, hi(TALER_Anz1)
		ld B, 8			; Geschwindigkeit
		ld A, 0FFh
TALER_f1:	call DAK2
		djnz TALER_f1
		cp 004h			; '0' gedrckt?
		jr Z, TALER_f2		;  --> ja, nach links
		cp 007h			; '3' gedrckt?
		jr Z, TALER_f3		;  --> ja, nach rechts
		jr TALER_f4
; Punkt nach links verschieben
TALER_f2:	ld A, 5
		cp L			; linke Grenze erreicht?
		jr Z, TALER_f4		;  --> ja
		res 4, (HL)
		inc HL			;  sonst Position anpassen
		set 4, (HL)
		jr TALER_f4
; Punkt nach rechts verschieben
TALER_f3:	ld A, 0
		cp L			; rechte Grenze erreicht?
		jr Z, TALER_f4		;  --> ja
		res 4, (HL)
		dec HL			;  sonst Position anpassen
		set 4, (HL)
TALER_f4:	ld A, L			; neue Position des Fngers eintragen
		ld (TALER_Pos), A
		ret
;
; Taler fallen lassen
; Prfung, ob im Anzeigebereich 1 irgendwo Bit 7 (unterer Balken) gesetzt ist
TALER_Sterne:	ld B, 7
		ld HL, TALER_Anz1
TALER_s1:	bit 7, (HL)
		call NZ, TALER_vergl
		res 7, (HL)
		inc HL
		djnz TALER_s1
; Prfung, ob im Anzeigebereich Bit 3 (mittlerer Balken) gesetzt ist,
; diesen lschen, Bit 7 setzen (Verschiebung nach unten)
		ld B, 7
		ld HL, TALER_Anz1
TALER_s2:	bit 3, (HL)
		jr Z, TALER_s3
		set 7, (HL)
		res 3, (HL)
TALER_s3:	inc HL
		djnz TALER_s2
; Prfung, auf Bit 2 (oberer Balken), diesen lschen, Bit 3 setzen
; (Verschiebung nach unten)
		ld B, 7
		ld HL, TALER_Anz1
TALER_s4:	bit 2, (HL)
		jr Z, TALER_s5
		set 3, (HL)
		res 2, (HL)
TALER_s5:	inc HL
		djnz TALER_s4
; an zuflliger Stelle neues Bit 2 (oberer Balken) setzen
		ld A, R
		and 007h
		ld L, A
		ld H,hi(TALER_Anz1)
		set 2, (HL)
		ret
;
; Prfen ob gefangen wurde
TALER_Vergl:	ld A, (TALER_Pos)	; Position Fnger
		cp L
		jr NZ, TALER_v2
		ld A, (TALER_Punkte01)
		add A, 1		; Punktezhler (Einer, Zehner) um 1 erhhen
		daa
		ld (TALER_Punkte01), A
		jr NC, TALER_v1		; bertrag?
		ld A, (TALER_Punkte10)
		add A, 1		;  ja, (Hunderter, Tausender) um 1 erhhen
		daa
		ld (TALER_Punkte10), A
TALER_v1:	push BC
		push HL
		push DE
		call SOU1K		; Ton bei Treffer
		pop DE
; die rechten beiden Stellen des Anzeigebreichs 2 lschen
		ld HL, TALER_Anz2
		ld (HL), 0
		inc HL
		ld (HL), 0
		pop HL
		pop BC
TALER_v2:	ret

;******************************************************************************



;******************************************************************************
; Programm 6
; Jagdspiel
; Autor: Volker Pohlers
;------------------------------------------------------------------------------
; Nach Begrung luft irgendwann eine Ziffer o. Buchstabe ber die Anzeige. 
; Entsprechende Taste drcken! Tonausgabe bei Tastendruck. Die Pos. des Treffers 
; wird ausgewertet. Endauswertung bei Taste "LD". Druck auf bel. Taste: Neubeginn.
;******************************************************************************

;
; Speicher
;
JAGD_z:		equ 02113h
JAGD_d:		equ 02115h
JAGD_rn:	equ 02117h


JAGD:		ld HL, 0000h
		ld (JAGD_d), HL
		ld (JAGD_z), HL
		ld (JAGD_rn), HL
;
; Begrung
;
		ld E, 3
JAGD_m0:	call JAGD_schuss10
		call JAGD_anzjagd
		dec E
		jr NZ, JAGD_m0
;
; RND-Zeitschleife
;
JAGD_m6:	call JAGD_rnd
JAGD_m2:	ld BC, 6			; 2
JAGD_m1:	djnz JAGD_m1
		dec C
		jr NZ, JAGD_m1
		dec A
		jr NZ, JAGD_m2
;
; Auswahl Ziffer JAGD_z
;
		call JAGD_rnd
		call ONESEG
;
; Wandern der Ziffer JAGD_z
;
		ld E, 5
		ld C, A
		ld IX, DATLED
JAGD_m5:	call JAGD_clranz
		ld D, 0
		add IX, DE
		ld (IX), C
		ld IX, DATLED
		ld B, 020h			; 10h
JAGD_m4:	call DAK2
		jr NC, JAGD_m3
JAGD_m9:	djnz JAGD_m4
		ld B, A
		dec E
		ld A, E
		cp A, 0FFh
		ld A, B
		jr NZ, JAGD_m5
;
; JAGD_z erhhen
;
JAGD_m11:	ld HL, (JAGD_z)
		inc HL
		ld (JAGD_z), HL
		jp JAGD_m6			; nchste Runde
;
; Auswertung Taste
;
JAGD_m3:	cp A, 002h			; Taste LD => Ende
		jp Z, JAGD_m7
		ld HL, KEYTAB
		add A, L
		ld L, A
		jr NC, JAGD_m8
		inc H
JAGD_m8:	ld A, (HL)
		call ONESEG
		call JAGD_schuss
		cp A, C
		jp NZ, JAGD_m9			; falsche Taste
;
; Berechnung Durchschnitt JAGD_d
;
		ld HL, (JAGD_d)
		ld A, E
		cp A, 0
		inc HL
		jr Z, JAGD_m10
		cp A,1
		inc HL
		jr Z, JAGD_m10
		cp A,2
		inc HL
		jr Z, JAGD_m10
		cp A, 3
		inc HL
		jr Z, JAGD_m10
		cp A, 4
		inc HL
		jr Z, JAGD_m10
		inc HL
JAGD_m10:	ld (JAGD_d), HL
		jp JAGD_m11
;
JAGD_m7:	call JAGD_schuss10
;
; Division JAGD_d / JAGD_z
;
		ld HL, (JAGD_d)
		ld DE, (JAGD_z)
		ld C, 0ffh
JAGD_m13:	inc C
		and A
		sbc HL, DE
		jr NC, JAGD_m13
		ld A, C
;
; Mult 6
;
		ld B, 5
JAGD_m12:	add A, C
		djnz JAGD_m12
;
; Auswahl Bewertung und Anzeige
;
		ld B, 0
		ld C, A
		ld IX, JAGD_Anz
		add IX, BC
		call DAK1
		jp JAGD
;
; Schussgerusch
;
JAGD_schuss:	push AF
		push BC
		push DE
		ld D, 3
		call JAGD_schussn
		pop DE
		pop BC
		pop AF
		ret
JAGD_schuss10:	ld D, 010h			; Anzahl der Schsse
JAGD_schussn:	ld C, 008h			; Startfrequenz (bzw -Periodendauer)
JAGD_q1:	ld B, 058h			; Anzahl der Perioden
JAGD_q1a:	call JAGD_schuss2		; Lautsprecher klicken lassen
		nop
		ld A, C				; Periodendauer holen
JAGD_q2:	dec A
		jr NZ, JAGD_q2			;  und abwarten
		inc C				; Periodendauer erhhen, d.h. Frequenz senken
		djnz JAGD_q1a			; nchste Periode
		dec D
		jr NZ, JAGD_schussn		; nchster Schuss
		ret
;
; Titel anzeigen fr 2,56 s
;		
JAGD_anzjagd:	ld IX, JAGD_Text
		ld B, 0
JAGD_q3:	call DAK2
		djnz JAGD_q3
		ret
;
; Anzeigebereich mit 000h fllen
;		
JAGD_clranz:	ld B, 6
		ld HL, DATLED
JAGD_q4:	ld (HL), 0
		inc HL
		djnz JAGD_q4
		ret
;
; Zufallszahl erzeugen
;		
JAGD_rnd:	ld DE, (JAGD_rn)
		ld A, 048h
		and D
		jp PO, JAGD_q5
		scf
JAGD_q5:	rl D
		res 7, D
		ld A, E
		rr E
		and 011h
		jp PO, JAGD_q6
		set 7, D
JAGD_q6:	ld A, D
		xor E
		ld (JAGD_rn), DE
		ret
;
JAGD_Text:	db 008h,0e9h,0e6h,06fh,0e1h,008h	; "-JAGD-"
JAGD_Anz:	db 011h,0cah,0e8h,020h,0e9h,021h	; "Idiot!"
		db 0cah,0cah,0e8h,0e6h,000h,0e7h	; "O Gott"
		db 000h,011h,0e9h,06fh,0eah,000h	; " Bad! "
		db 011h,06fh,0e1h,000h,06fh,068h	; "Na Ja!"
		db 000h,0e9h,0e8h,0e8h,0e6h,000h	; " Good "
		db 011h,048h,0ceh,04fh,0e3h,0aeh	; "Super!"
		db 048h,0ceh,0cah,0aeh,0ceh,0eah	; "Bester"
;
JAGD_schuss2:	in A, DIGITAP
		xor 002h			; Lautsprecher umschalten
		out DIGITAP, A
		ret

;******************************************************************************



;******************************************************************************
; Programm 7 / 8
; U2716 (EPROM-Programmierung)
; Autor: Gunther Zielosko
; "Hinweise zur Anwendung des Lerncomputers LC 80", Teil 2, Seite 17-28
;------------------------------------------------------------------------------
; Lesen: Soll der Inhalt eines programmierten EPROMs (oder ROMs) in den RAM-
; Speicher den LC-80 ab Adresse 2400 gelesen werden, wird 'LD' bettigt, nach
; Ausfhrung erscheint "READY" (ca. 2 s).
; Schreiben: Soll ein EPROM geladen werden, mssen natrlich die dafr
; bestimmten Daten im RAM-Bereich ab 2400 stehen. Nach dem Drcken von 'ST'
; erfolgt automatisch eine Leerkontrolle des zu programmierenden EPROMs. Ist
; der EPROM leer, erscheint kurz die Anzeige "LEER" und dann wird sofort
; programmiert. Ist er nicht vollstndig leer, erscheint "ERROR"
;  - es erfolgt dann keine Programmierung.
;  - Beim eigentlichen Programmiervorgang knnen Adressen und Daten beobachtet
;    werden. Eine erfolgreiche Programmierung wird mit "PASS", ein Fehler mit
;    "FAIL" quittiert. Nach insgesamt 5 Programmierzyklen wird "READY"
;    angezeigt.
;
; Eigene Speicherbereiche festlegen: 
;  - 'RES', 'ST' bettigen, die Anzeige ist dann " X.X.X.X.-F " (X bedeutet
;    hier, da die dort angezeigten Werte ohne Bedeutung sind).
;  - ber die Tastatur wird jetzt die Anzahl der zu bertragenden Bytes in
;    hexadezimaler Form eingegeben. Bei 72 Bytes (hex.) ist die abschlieende
;    Anzeige dann " 0.0.7.2.-F "
;  - '+' drcken, es erscheint die Anzeige " X.X.X.X.-S "
;  - ber die Tastatur wird die Startadresse im LC-80 eingegeben (meist im
;    RAM-Bereich, vorzugsweise ab 2400; es ist aber genauso mglich, einen
;    ROM-Bereich in einen EPROM zu bertragen - aber nicht umgekehrt!).
;    Nach der Eingabe z. B. der Startadresse 2000 ist die Anzeige dann 
;    " 2.0.0.0.-S "
;  - '+' drcken, es erscheint die Anzeige " X.X.X.X.-E "
;  - Jetzt Eingabe der Startadresse im EPROM, der zu programmieren ist. Die
;    EPROM-Adressen beginnen grundstzlich bei 0000 und enden beim U 2716 C
;    bei 07FF. Soll z. B. der EPROM-Bereich ab 01B0 beladen werden, mu auf
;    dem Display zu lesen sein: " 0.1.B.0.-E "
;  - 'RES', 'ADR' bettigen
;  - ber '1' , '0' , '0' , '0' , 'EX' das Men aurufen, Programm '7' whlen
;  - Danach kann wie blich mit 'ST' programmiert und mit 'LD' gelesen werden
;******************************************************************************

; Hauptprogramm
;
; Das Hauptprogramm organisiert den gesamten Programmierablauf einschlielich der
; Funktionen:
; - Leerkontrolle (Ist EPROM vollstndig EPR_LEER? - FF?)
; - Einlesen eines programmierten EPROMs in den RAM-Bereich
; - Vergleich Ist- und Sollwert jeder Zelle.
;
U2716:		LD HL, EPR_memad	; Standardwerte in Parameterspeicher
		LD DE, 023EBh		; fr Kassettenbetrieb laden
		LD BC, 00006h
		LDIR
; Hier Einsprung wenn eigene Speicherbereiche definiert wurden	
U2716_1:	CALL EPR_INIT		; Initialisierung Port B
EPR_H1:		CALL EPR_AUS		; Grundzustand: VCC,VPP = 0 V
		LD IX, EPR_aU2716	; Anzeige "U 2716"
		CALL DAK1
		CP A, 01Fh		; 'LD' gedrckt?
		PUSH AF
		CALL Z, EPR_LESEN	;  ja --> lesen
		POP AF
		CP A, 01Eh		; 'ST'gedrckt?
		JR NZ, EPR_H1		;  nein --> weiter
		CALL EPR_LEER		;  sonst Leertest ausfhren
		LD B, 5			; 5 Programmierzyklen
EPR_H2:		PUSH BC
		CALL EPR_PROGR		; Programmierung ausfhren	
		CALL SOU1K		; Ton ausgeben
		CALL EPR_VERGL		; Ergebnis prfen
		POP BC
		DJNZ EPR_H2
		PUSH AF			; Vergleichsergebnis sichern
		CALL EPR_AUS		; Grundzustand: VCC,VPP = 0 V
		POP AF			; Vergleichsergebnis zurckholen
		JR C, EPR_H3		;  ok --> weiter
; Fehler beim Programmieren
EPR_H4:		LD IX, TXT_ERR		; Anzeige "ERROR"
		CALL DAK1
		JR EPR_H1
; Programmierung erfolgreich
EPR_H3:		LD IX, EPR_aREADY	; Anzeige "READY"
		CALL DAK1
		JR EPR_H1
;
; Unterprogramm EPR_INIT
;
; Das Unterprogramm EPR_INIT dient zur Initialisierung des Port B der USER-PIO.
;
EPR_INIT:	LD A, 0FFh		; PIO-Port B, Mode 3
		OUT PIO2BC, A
		LD A, 0F0h 		; E/A-Definition:
		OUT PIO2BC, A		; B0 .. B3 = Ausgnge, B4 .. B7 = Eingnge
		RET
		NOP
;
; Unterprogramm EPR_LESEN
;
; Das Unterprogramm EPR_LESEN realisiert die bernahme des Inhaltes des auf der
; Programmierfassung steckenden EPROMs U 2716 C in den RAM-Speicher des LC-80 (2.
; und 3. RAM, Adressen 2400 bis 2BFF). Nach AbscHLu des Vorganges erscheint die
; Anzeige "READY".
;
EPR_LESEN:	CALL EPR_ADVOR
EPR_L1:		CALL EPR_ADINK
		CALL EPR_DATIN
		LD (IY+00h), A
		INC IX
		CALL EPR_ENDE
		JR NZ, EPR_L1
		LD IX, EPR_aREADY	; Anzeige "READY"
		CALL DAK1
		RET
;
; Unterprogramm EPR_LEER
;
; Das Unterprogramm EPR_LEER vergleicht einzeln jedes Byte des auf der
; Programmierfassung steckenden EPROMs mit FF. Sind alle Zellen EPR_LEER, wird
; "EPR_LEER" angezeigt, ansonsten "ERROR".
;
EPR_LEER:	CALL EPR_ADVOR
EPR_O1:		CALL EPR_ADINK
		CALL EPR_DATIN
		CP A, 0FFh		; Vergleich mit FF
		JR NZ, EPR_O2
		CALL EPR_ENDE
		JR NZ, EPR_O1
		LD IX, EPR_aLEER	; Anzeige "EPR_LEER"
		LD B, 0FFh		; fr 255x 10 ms ausgeben 
EPR_O3:		CALL DAK2
		DJNZ EPR_O3
		RET
; Abbruch bei Fehler		
EPR_O2:		LD HL, EPR_H4
		EX (SP), HL
		RET
;
; Unterprogramm EPR_PROGR
;
; Im Unterprogramm EPR_PROGR wird der Inhalt von RAM 2 und 3 byteweise in den auf
; der Programmierfassung steckenden EPROM geladen. Nach AbscHLu erfolgt die Anzeige
; "READY". Pro Byte wird ein Programmierimpuls von ca. 10 ms erzeugt. Diese Zeit
; wird jedesmal durch Aufruf von DAK2 realisiert, ein Nebeneffekt ist die Anzeige
; von Adressen und Daten auf dem Display.
;
EPR_PROGR:	LD IX, 023F2h		; Monitor-Variable DATLED
		CALL EPR_ADVOR
		IN A, KEYIN
		SET 3, A		; /OE => 1   Programmier-
		RES 2, A		; /CE => 0   sperre
		OUT KEYIN, A
EPR_P1:		CALL EPR_ADINK
		LD A, (IY+00h)
		OUT USERPORT, A
		CALL DADP
		LD HL, 023F8h		; Monitor-Variable USERPC
		LD E, (HL)
		INC HL
		LD D, (HL)
		DEC DE
		CALL ADRSDP
		IN A, KEYIN
		SET 2, A		; /CE => 1
		OUT KEYIN, A
		CALL DAK2		; Programmierimpuls ca. 10 ms
		IN A, KEYIN
		RES 2, A		; /CE => 0
		OUT KEYIN, A
		INC IY
		CALL EPR_ENDE
		JR NZ, EPR_P1
		RET
;
; Unterprogramm EPR_VERGL
;
; Das Unterprogramm EPR_VERGL organisiert einen Bitmustervergleich zwischen EPROM
; und Zusatz-RAM. Bei bereinstimmung erscheint "PASS", bei Nichtbereinstimmung
; "FAIL".
;
EPR_VERGL:	CALL EPR_ADVOR
EPR_V1:		CALL EPR_ADINK
		CALL EPR_DATIN
		LD B, (IY+00h)
		INC IY
		CP A, B
		JR NZ, EPR_V2
		CALL EPR_ENDE
		JR NZ, EPR_V1
		LD IX, EPR_aPASS-1	; Anzeige "PASS"
		SCF
		JR EPR_V3
EPR_V2:		LD IX, EPR_aFAIL	; Anzeige "FAIL"
		OR A			; CY: = 0
EPR_V3:		LD B, 0FFh		; Anzeigezeit 2,55 s
EPR_V4:		PUSH AF
		CALL DAK2
		POP AF
		DJNZ EPR_V4
		RET
;
; Unterprogramm EPR_ADVOR
;
; EPR_ADVOR dient zur Herstellung der Anfangsbedingungen bei der Adressierung von
; RAM und EPROM sowie zur Nullsetzung des Byte-ZHLers.
;
EPR_ADVOR:	LD HL, 023FEh		; Monitor-Variable USERPC (Bytezhler)
		LD (HL), 000h
		INC HL
		LD (HL), 000h
		LD HL, 023EDh		; Monitor-Variable FUFEFLAG
		LD E, (HL)
		INC HL
		LD D, (HL)
		PUSH DE
		POP IY
		INC HL
		LD DE, 023F8h		; Monitor-Variable USERPC
		LD BC, 00002h
		LDIR
		LD A, 007h
		OUT KEYIN, A
		RET
;
; Unterprogramm EPR_ADINK
;
; Das Unterprogramm EPR_ADINK organisiert die Erhhung der Adressen von RAM und EPROM
; sowie des Byte-ZHLers, bis der vorgegebene Betrag (2 KByte) erreicht wird.
;
EPR_ADINK:	LD A, 00Fh
		OUT PIO2AC, A
		LD HL, 023F8h		; Monitor-Variable USERPC
EPR_ADNK1:	IN A, KEYIN		; Im Unterprogramm EPR_AUS wird zu dieser Marke gesprungen.
		RES 0, A
		OUT KEYIN, A
		LD A, (HL)
		LD C, A
		OUT USERPORT, A
		INC HL
		IN A, KEYIN
		SET 0, A
		RES 1, A
		OUT KEYIN, A
		LD A, (HL)
		LD B, A
		OUT USERPORT, A
		INC BC
		LD (HL), B
		DEC HL
		LD (HL), C
		IN A, KEYIN
		SET 1, A
		OUT KEYIN, A
		LD HL, 023FEh		; Monitor-Variable USERPC (Bytezhler)
		LD E, (HL)
		INC HL
		LD D, (HL)
		INC DE
		LD (HL), D
		DEC HL
		LD (HL), E
		RET
;
; Unterprogramm EPR_DATIN
;
; Das Unterprogramm EPR_DATIN ist zustndig fr das Einlesen eines EPROM-Bytes in
; das entsprechende RAM-Byte bei vorher organisierter Adressierung.
;
EPR_DATIN:	IN A, KEYIN
		RES 2, A
		OUT KEYIN, A
		LD A, 04Fh
		OUT PIO2AC, A
		IN A, USERPORT
		PUSH AF
		IN A, KEYIN
		SET 2, A
		OUT KEYIN, A
		POP AF
		RET
;
; Unterprogramm EPR_AUS
;
; Bei EPR_AUS wird auf allen PIO-Leitungen 0 ausgegeben, damit werden auch UCC und
; UPR abgeschaltet, ebenso /CE und /OE.
;
EPR_AUS:	LD A, 00Fh
		OUT PIO2AC, A
		LD HL, 098Ah-ROMOFS	; Bei Bestckung des LC-80 mit 2 ROMs U 505 D ist hier einzutragen: LD HL, 098A
		CALL EPR_ADNK1		; Hier wird nicht EPR_ADINK angesprungen, sondern die Marke 1 in diesem Unterprogramm!
		LD A, 000h
		OUT KEYIN, A
		RET
;
; Unterprogramm EPR_ENDE
;
; Das Unterprogramm EPR_ENDE vergleicht den Byte-Zhler mit der Anzahl zu programmierender Bytes.
; Je nach Ergebnis wird das Zero-Flag gesetzt. Bei Z = 1 sind alle Bytes
; abgearbeitet.
;
EPR_ENDE:	LD HL, 023FEh		; Monitor-Variable FILE2
		LD C, (HL)
		INC HL
		LD B, (HL)		; Byte-Zhler in BC
		LD HL, 023EBh		; Monitor-Variable FILNAME 
		LD E, (HL)
		INC HL
		LD D, (HL)		; Lnge in DE
		EX DE, HL
		OR A			; CY = 0 setzen, Z-Flag setzen, wenn EPR_H1 = BC
		SBC HL, BC
		RET
;
; Tabelle der Anzeigetexte und Ladeparameter
;
EPR_memad:	DW 00800h	; Anzahl der zu bertragenden Bytes (800 hexadezimal - 2048 dezimal - 2 KByte)
		DW 02400h	; Anfangsadresse im RAM-Bereich (2400)
		DW 00000h	; Anfangsadresse beim zu programmierenden EPROM U 2716 C (0000)
;
EPR_aPASS:	DB 0AEh, 0AEh, 06Fh, 04Fh, 000h			; "PASS"
EPR_aREADY:	DB 000h, 0ABh, 0E9h, 06Fh, 0CEh, 048h, 000h	; "READY"
EPR_aLEER:	DB 000h, 048h, 0CEh, 0CEh, 0C2h, 000h		; "LEER"
EPR_aU2716:	DB 0EEh, 021h, 025h, 0CDh, 000h, 0E3h		; "U 2716"
EPR_aFAIL:	DB 000h, 0C2h, 021h, 06Fh, 04Eh, 000h		; "FAIL"
;
		DB 0FFh

;******************************************************************************



;******************************************************************************
; Programm 9
; LLC2-RAM-Test
; testet zusammenhngenden RAM ab 2000
; Anzeige Adresse+Inhalt erste fehlerhafte/nicht-RAM-Zelle
; dann beliebige Taste fr Warmstart
;******************************************************************************

RAMTEST:	ld BC, 00080h		; max. 32k prfen
		ld HL, 02000h 		; Startadresse RAM
RAM_SL:		call RAM_CHK		; Zelle prfen
        	jr NZ, RAM_ENDE		; Fehler, Anzeige letzte RAM-Adresse
		inc HL			; nchste Adresse
		djnz RAM_SL		; 256 Bytes prfen
		dec C			; nchster Block
		jr NZ,RAM_SL
; fertig oder Fehler
RAM_ENDE:	push HL			; Adresse Fehlerzelle
		ld A, (HL)		; deren Inhalt
		call DADP 		; letzte Daten in Anzeige
		pop HL
		ld D, H
		ld E, L
		call ADRSDP		; Adresse in Anzeige
		ld IX, DATLED		; Standard-Anzeigespeicher
		call DAK1		; anzeigen bis Tastendruck
		jp MENUE		         
;
; Speicherzelle prfen
RAM_CHK:	LD A, (HL)		; alten Wert holen
		CPL 			; Komplement
		LD (HL), A		; schreiben
		LD A, (HL)		; geschriebenen Wert lesen
		CPL			; Komplement
		LD (HL), A		; schreiben
		CP (HL)			; geschrieben = gelesen?
		RET

;******************************************************************************

	END
